package com.example.aped;

import com.example.aped.utils.XMLHandler;

import org.junit.Test;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.IOException;
import java.util.Dictionary;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import static org.junit.Assert.*;

public class XMLHandlerUnitTest {
    private String xmlPath="src/main/assets/XML/Test.xml";
    private File xmlFile = new File(xmlPath);

    @Test
    public void TestFiles_AreValid(){
        try{
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
        }catch(VerifyError e){
            System.out.println("XML not valid");
            assert(false);
        }
    }

    @Test
    public void Test_getDeviceNames(){
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
            List<String> deviceNames = xmlHandler.getDeviceNames();
            assertArrayEquals(new String[]{"example","PWM_example","sensorarray"},deviceNames.toArray());
    }

    @Test
    public void TestInput_SimplePort(){
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
            Dictionary<String, Object> port = xmlHandler.getPort("example");
            assertEquals("{pins=[GPIO_2], frequency=, protocol=DI}",port.toString());
    }

    @Test
    public void TestInput_ComplexPort(){
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
            Dictionary<String, Object> port = xmlHandler.getPort("sensorarray");
            assertEquals("{pins=[GPIO_3, GPIO_4, GPIO_5, GPIO_6], frequency=, protocol=DI}",port.toString());
    }

    @Test
    public void TestInput_BufferSize(){
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
            int bufferSize = xmlHandler.getBufferSize("PWM_example");
            assertEquals(1000,bufferSize);
    }

    @Test
    public void TestInput_NoBufferSize(){
        XMLHandler xmlHandler = new XMLHandler(xmlFile);
        int bufferSize = xmlHandler.getBufferSize("example");
        assertEquals(0,bufferSize);
    }
}
